function [ G, GSpat, GSpec ] = GSharp(k, x1, x2, d, alpha)
% Compute the gradient of the 1-periodic Green's function Gsharp (no Dirichlet boundary condition)
% CAUTION : Only normal incident waves are supported in this version
% In :
%   k : the wave number. k = 0 means Laplace
%   x1 : a 1xN vector containing the first coordinates
%   x2 : a 1xN vector containing the second coordinates
% OUT
%   returns G_\sharp(x1,x2).

% assert (k >= 0, 'k must be real (for now)');

kx0 = alpha;
P = 5;
M = 5;
Q = 5;

% E1
E1 = sqrt(pi)/d; 
% E2
kz0 = sqrt(k^2 - kx0^2);
Hsqrd = 9;
H = sqrt(Hsqrd);
E2 = kz0/(2*H);
% E3
epsi = 1e-5;
E3 = k/(2*(epsi*factorial(Q))^(1/2/Q));
E = max([E1, abs(E2), abs(E3)]);



%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Calculation of Gspectral
%%%%%%%%%%%%%%%%%%%%%%%%%%%%
erfcz = @(z) 1 - ops.erfz(z);

GSpec = 0;
for p=-P:P
    kxp = kx0 + 2*pi*p/d;

%     if k^2 > kxp^2
%         kzp = sqrt(k^2 - kxp^2);
%     else
%         kzp = 1i * sqrt(kxp^2 - k^2);
%     end
    
%     branchCutShift = 0;
    
%     if  p ~= 0
        branchCutShift = -pi/2;
%     end    
    
    e1 = exp(-branchCutShift*1i);
    e2 = exp(branchCutShift/2*1i);
    kzp = sqrt((k^2 - kxp^2)*e1)*e2;
    
    term1 = 1/4/d * exp(-1i*kxp*x1)/1i/kzp;
    term2 = exp(1i*kzp*abs(x2)).*erfcz(1i*kzp/2/E + abs(x2)*E);
    term3 = exp(-1i*kzp*abs(x2)).*erfcz(1i*kzp/2/E - abs(x2)*E);
    
%     fprintf('term3: %.4f\n', term3);
    
%     GSpec = GSpec + term1.*(term2+term3);
    GSpec = GSpec + term1.*(term2+term3);

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Calculation of Gspatial
%%%%%%%%%%%%%%%%%%%%%%%%%%%%
GSpat = 0;
for m=-M:M
    Rm = sqrt(x2.^2 + (x1-m*d).^2);
    
    term1 = 0;
    Eqp1 = ops.E1((Rm*E).^2);
    for q=0:Q
        term1 = term1 + (k/2/E).^(2*q)./factorial(q).*Eqp1;
        Eqp1 = (exp(-(Rm*E).^2) - (Rm*E).^2.*Eqp1)./(q+1);
    end
    
    term1 = 1/4/pi.*exp(-1i*kx0*m*d).*term1;
%     if any(isnan(1/4/pi.*exp(-1i*kx0*m*d).*term1))
%         term1(isnan(term1)) = 0;
%     end
    
    GSpat = GSpat + term1;
end

G = GSpat + GSpec;
end
