classdef Kstar_H_Periodic < ops.Operators
    
    methods
        function obj = Kstar_H_Periodic(k, D, type, step, d, alpha)
            obj = obj@ops.Operators(D, type, step, D, type, step);
            
            obj.Kmat = ops.Kstar_H_Periodic.make_kernel_matrix(k, D.points, D.tvec, D.normal, D.avec, D.sigma, d, alpha);
        end
        
    end
    
    methods(Static)
        function K = make_kernel_matrix(k, D, tvec, normal, avec, sigma, d, alpha)
            N = size(D,2);
            K = zeros(N, N);
              
            for i = 1:N
                grad = -conj(ops.GradGSharp(conj(k), D(1,i)-D(1,:), D(2,i)-D(2,:), d, alpha));
                K(i, :) = (normal(:,i)'*grad).*sigma;
             
                if i == 1
                    K(i,i) = 1/2 * (K(i, N) +  K(i, 2));
                elseif i == N
                    K(i,i) = 1/2 * (K(i, i-1) +  K(i, 1));
                else
                   K(i,i) = 1/2 * (K(i, i-1) +  K(i, i+1));
                end                
                
%                 K(j,j) = -1/4/pi*avec(:,j)'*normal(:,j)/tvec_norm_square(j)*sigma(j);
            end
            %%%%%%%%%%%%%%%%%%%%%%%%
        end
        
        function [ val ] = eval(D, F)
            % Evaluate K_D^*[F]
            % INPUTS:
            % D: shape, a C2boundary object
            % F: function defined on D
            
            Ks = ops.Kstar.make_kernel_matrix(D.points, D.tvec, D.normal, D.avec, ...
                D.sigma);
            val = Ks*F(:);
        end
        
    end
end

