function res = CalculateResonance(p, a, nPoints, vB, delta, enableDirichlet, fixedDistBoundary, distBoundary)
% Calculate and plot the resonant frequency given by the formula over a
% range of periods.
B = shape.Ellipse(a, a, nPoints);

% Select the appropriate layer potentials depending on whether we have a
% Dirichlet boundary condition
if enableDirichlet == 1
    % Use either a fixed distance to the boundary, or else scale the
    % distance based on the bubble radius
    if fixedDistBoundary == 1
        B = B + [ 0 ; distBoundary ];
    else
        B = B + [ 0 ; a*distBoundary ];
    end
    Ks = ops.Kstar_PeriodicDirichlet(B, 'P0', 1, p);
    S = ops.SingleLayer_PeriodicDirichlet(B, 'P0', 1, p);
else
    Ks = ops.Kstar_Periodic(B, 'P0', 1, p);
    S = ops.SingleLayer_Periodic(B, 'P0', 1, p);
end
Ks = Ks.stiffmat;
S = S.stiffmat;

% Locate the position of the eigenvalue 0.5 and retrieve the
% corresponding eigenfunction (or is it always in the same place?) and
% then normalize it
[V_Ks, D_Ks] = eig(Ks);
[diff, index] = min(abs(D_Ks(:) - 0.5));
colIndex = mod(index, nPoints);
eigFuncKs = V_Ks(1:nPoints,colIndex:colIndex);
eigFuncKs = eigFuncKs/dot(eigFuncKs, B.sigma);

% Determine the capacity and volume
S = S * eigFuncKs;
capacity = -1/S(1);
% capacity = exp(2*pi*S);
vol = pi*a^2;

res = sqrt(delta * capacity/vol) * 1/vB;

end